package Genealogy::Wills::wills;

use strict;
use warnings;
use Database::Abstraction;

=head1 NAME

Genealogy::Wills::wills - database driver for Genealogy::Wills

=head1 VERSION

Version 0.06

=cut

our $VERSION = '0.06';

our @ISA = ('Database::Abstraction');

1;
